"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const dotenv_1 = require("dotenv");
dotenv_1.config();
require("enve");
const finish_1 = require("lib/login/passwordless/finish");
const start_1 = require("lib/login/passwordless/start");
const finish_2 = require("lib/register/finish");
const start_2 = require("lib/register/start");
const set_password_1 = require("lib/account/set-password");
const check_email_1 = require("lib/register/check-email");
const get_1 = require("lib/account/get");
const clean_1 = require("lib/email/clean");
const to_id_1 = require("lib/email/to-id");
const verify_1 = require("lib/jwt/verify");
const storage = require("node-persist");
const set_totp_1 = require("lib/account/set-totp");
const sign_1 = require("lib/jwt/sign");
const login_1 = require("lib/login/login");
const speakeasy_1 = require("speakeasy");
require("jest-extended");
const { TEST_STORAGE } = process.enve;
beforeAll(async () => {
    try {
        const userId = await to_id_1.emailToId('test@example.com');
        await storage.init(TEST_STORAGE);
        await storage.removeItem(`user-${userId}`);
        await storage.removeItem(`email-test@example.com`);
    }
    catch (err) {
        return;
    }
});
test('clean email', () => {
    expect(clean_1.cleanEmail('e.m.a.il+test@gmail.com')).toBe('email@gmail.com');
    expect(clean_1.cleanEmail('e.mail+t@example.com')).toBe('e.mail+t@example.com');
});
test('check email (available, does not exist)', async () => {
    const { available } = await check_email_1.checkEmail('test@example.com');
    expect(available).toBeTrue();
});
test('sign and verify jwt', async () => {
    const encoded = await sign_1.signJWT(1234, 'test@example.com', '1m');
    const decoded = await verify_1.verifyJWT(encoded);
    expect(decoded.userId).toBe(1234);
    expect(decoded.email).toBe('test@example.com');
});
test('start registration', async () => {
    const token = await start_2.startRegistration('test@example.com', 'test1234');
    await expect(verify_1.verifyJWT(token)).not.toReject();
});
test('email to user id', async () => {
    const userId = await to_id_1.emailToId('test@example.com');
    expect(userId).toBeNumber();
    expect(userId.toString()).toMatch(/^\d{13}$/);
});
test('check email (available, exists but not verified)', async () => {
    const { available } = await check_email_1.checkEmail('test@example.com');
    expect(available).toBeTrue();
});
test('finish registration', async () => {
    const tempToken1 = await start_2.startRegistration('test@example.com', 'test1234');
    const tempToken2 = await start_2.startRegistration('test@example.com', 'test1234');
    await expect(finish_2.finishRegistration(await verify_1.verifyJWT(tempToken1))).toReject();
    const fullToken = await finish_2.finishRegistration(await verify_1.verifyJWT(tempToken2));
    await expect(verify_1.verifyJWT(fullToken)).not.toReject();
}, 10000);
test('check email (not available)', async () => {
    const { available } = await check_email_1.checkEmail('test@example.com');
    expect(available).toBeFalse();
});
test('login (no totp)', async () => {
    const token = await login_1.login('test@example.com', 'test1234');
    await expect(verify_1.verifyJWT(token)).not.toReject();
});
test('get account', async () => {
    const userId = await to_id_1.emailToId('test@example.com');
    const account = await get_1.getAccount(userId);
    expect(account.email).toBe('test@example.com');
    expect(account.hasPassword).toBeTrue();
    expect(account.hasTOTP).toBeFalse();
});
test('set totp', async () => {
    const userId = await to_id_1.emailToId('test@example.com');
    let data = await set_totp_1.setTOTP(userId, false);
    expect(data.secret).toBeUndefined();
    expect(data.url).toBeUndefined();
    data = await set_totp_1.setTOTP(userId, true);
    expect(data.secret).toBeString();
    expect(data.secret).toHaveLength(32);
    expect(data.url).toBeString();
    const otp = speakeasy_1.totp({ secret: data.secret });
    expect(speakeasy_1.totp.verify({ secret: data.secret, token: otp })).toBeTrue();
});
test('login (with totp)', async () => {
    const userId = await to_id_1.emailToId('test@example.com');
    const data = await set_totp_1.setTOTP(userId, true);
    const otp = speakeasy_1.totp({ secret: data.secret });
    const token = await login_1.login('test@example.com', 'test1234', otp);
    await expect(verify_1.verifyJWT(token)).not.toReject();
});
test('passwordless login', async () => {
    const tempToken = await start_1.startPasswordlessLogin('test@example.com');
    await expect(verify_1.verifyJWT(tempToken)).not.toReject();
    const fullToken = await finish_1.finishPasswordlessLogin(await verify_1.verifyJWT(tempToken));
    await expect(verify_1.verifyJWT(fullToken)).not.toReject();
});
test('remove password', async () => {
    const userId = await to_id_1.emailToId('test@example.com');
    await expect(set_password_1.setPassword(userId, null)).not.toReject();
    const account = await get_1.getAccount(userId);
    expect(account.hasPassword).toBeFalse();
});
//# sourceMappingURL=tests.js.map